@@ -10,6 +10,7 @@ package Debconf::DbDriver::Directory;
 use strict;
 use Debconf::Log qw(:all);
 use IO::File;
+use POSIX;
 use Fcntl qw(:DEFAULT :flock);
 use Debconf::Iterator;
 use base 'Debconf::DbDriver::Cache';
@@ -18,12 +19,12 @@ use base 'Debconf::DbDriver::Cache';
 
 This is a debconf database driver that uses a plain text file for
 each individual item. The files are contained in a directory tree, and
-are named according to item names, with slashes replaces by colons.
+are named according to item names, with slashes replaced by colons.
 
 It uses a Format module to handle reading and writing the files, so the
 files can be of any format.
 
-This is foundation for other DbDrivers, and it not itself useable as one.
+This is a foundation for other DbDrivers, and is not itself usable as one.
 
 =head1 FIELDS
 
@@ -92,8 +93,11 @@ sub init {
 		# dropped when this object is destroyed.
 		open ($this->{lock}, ">".$this->{directory}."/.lock") or
 			$this->error("could not lock $this->{directory}: $!");
-		flock($this->{lock}, LOCK_EX | LOCK_NB) or
-			$this->error("$this->{directory} is locked by another process");
+		while (! flock($this->{lock}, LOCK_EX | LOCK_NB)) {
+			next if $! == &POSIX::EINTR;
+			$this->error("$this->{directory} is locked by another process: $!");
+			last;
+		}
 	}
 }
 
@@ -147,14 +151,15 @@ sub save {
 		open($fh, ">$file-new") or $this->error("$file-new: $!");
 	}
 	$this->{format}->beginfile;
-	$this->{format}->write($fh, $data, $item);
+	$this->{format}->write($fh, $data, $item)
+		or $this->error("could not write $file-new: $!");
 	$this->{format}->endfile;
 	
 	# Ensure it is synced, to disk buffering doesn't result in
 	# inconsistencies.
-	$fh->flush;
-	$fh->sync;
-	close $fh;
+	$fh->flush or $this->error("could not flush $file-new: $!");
+	$fh->sync or $this->error("could not sync $file-new: $!");
+	close $fh or $this->error("could not close $file-new: $!");
 	
 	# Now rename the old file to -old (if doing backups),
 	# and put -new in its place.
